/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/


#include "spi2_hw.h"
#include "pic_serial.h"
#include "debug.h"

void spi2_hw_setup_io() {


	make_output(PORTB, 3);	//sdo2
	make_input(PORTB, 2);	// sdi2


#ifdef SPI2_HW_MASTER_MODE
	

	make_output(PORTB, 1);	//sck2

		
	clear_bit(ssp2con1, SSPM3);
	clear_bit(ssp2con1, SSPM2);
	#ifdef SPI2_HW_MASTER_CLOCK_TMR2_DIV_2
		set_bit(ssp2con1, SSPM1);
		set_bit(ssp2con1, SSPM0);
	#endif
	#ifdef SPI2_HW_MASTER_CLOCK_FOSC_DIV_64
		set_bit  (ssp2con1, SSPM1);
		clear_bit(ssp2con1, SSPM0);
	#endif
	#ifdef SPI2_HW_MASTER_CLOCK_FOSC_DIV_16
		clear_bit(ssp2con1, SSPM1);
		set_bit  (ssp2con1, SSPM0);
	#endif
	#ifdef SPI2_HW_MASTER_CLOCK_FOSC_DIV_4
		clear_bit(ssp2con1, SSPM1);
		clear_bit(ssp2con1, SSPM0);
	#endif
#else
	// Slave mode
	make_input(PORTB, 1);	// sck
	
	clear_bit(ssp2con2, SSPM3);
	set_bit  (ssp2con2, SSPM2);
	clear_bit(ssp2con2, SSPM1);
	#ifdef SPI2_HW_USE_SS
		make_input(PORTB,0);	// SS2
		clear_bit(ssp2con2, SSPM0);
	#else
		set_bit(ssp2con2, SSPM0);
	#endif
#endif

}	

   
void spi2_hw_init() {
	
	#ifdef SPI2_HW_CLK_IDLE_IS_HIGH
		set_bit(ssp2con2, CKP);
	#else	
		// for 0,0 mode - mrf24j40 needs this
		clear_bit(ssp2con2, CKP);
	#endif
	#ifdef SPI2_HW_MASTER_MODE	

		set_bit(ssp2stat, SMP);	// from tim
	#else
		clear_bit(ssp2stat, SMP);	// smp must be cleared in slave mode
	#endif	
	
	#ifdef SPI2_HW_TRANSMIT_ON_IDLE_TO_ACT
		clear_bit(ssp2stat, CKE);
	#else
		// need this for mrf24j40
		// for 0,0 mode
		set_bit(ssp2stat, CKE);
	#endif	
	set_bit(ssp2con1, SSPEN);	// enable mssp serial port
}

	

void spi2_hw_master_transmit(uns8 data) {
	start_crit_sec();
	
	clear_bit(pir3, SSP2IF);
	// Try and send, if we have something already sending, try again
	do {
		clear_bit(ssp2con1, WCOL);
		ssp2buf = data;
	} while (test_bit(ssp2con1, WCOL));
	
	// Wait here while we transmit
	// Note that we will hang here if we don't have SPI setup properly
	// or we're running under the simulator
	while (!test_bit(pir3, SSP2IF)) {}
	end_crit_sec();
}

uns8 spi2_hw_master_receive() {
uns8 x;
	spi2_hw_master_transmit(0x00);	// dummy transmit to get something back
	x = ssp2buf;
	debug_str(" r=");
	debug_int_hex(x);
	return x;
	
}
	
void spi2_hw_reset(void) {

	clear_bit(ssp2con1, SSPEN);	// clear bit counter
	set_bit(ssp2con1, SSPEN);	// enable mssp serial port

}
